Module socketioxide::packet

source ·
Socket.io packet implementation. The Packet is the base unit of data that is sent over the engine.io socket. It should not be used directly except when implementing the Adapter trait.

Structs

  • Binary packet used when sending binary data
  • Connect packet sent by the client
  • The socket.io packet type. Each packet has a type and a namespace

Enums