FromDisconnectParts

Trait FromDisconnectParts 

pub trait FromDisconnectParts<A: Adapter>: Sized {
    type Error: Error + 'static;

    // Required method
    fn from_disconnect_parts(
        s: &Arc<Socket<A>>,
        reason: DisconnectReason,
    ) -> Result<Self, Self::Error>;
}
A trait used to extract the arguments from the disconnect event. The Result associated type is used to return an error if the extraction fails, in this case the DisconnectHandler is not called.

  • See the disconnect module doc for more details on disconnect handler.
  • See the extract module doc for more details on available extractors.

Required Associated Types

type Error: Error + 'static

The error type returned by the extractor

Required Methods

fn from_disconnect_parts( s: &Arc<Socket<A>>, reason: DisconnectReason, ) -> Result<Self, Self::Error>

Extract the arguments from the disconnect event. If it fails, the handler is not called

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors

Source

impl<A: Adapter> FromDisconnectParts<A> for ProtocolVersion

Source

impl<A: Adapter> FromDisconnectParts<A> for TransportType

Source

impl<A: Adapter> FromDisconnectParts<A> for DisconnectReason

Source

impl<A: Adapter> FromDisconnectParts<A> for SocketRef<A>

Source

impl<A: Adapter> FromDisconnectParts<A> for SocketIo<A>

Source

impl<A: Adapter, T: Clone + Send + Sync + 'static> FromDisconnectParts<A> for Extension<T>

Available on crate feature extensions only.
Source

impl<A: Adapter, T: Clone + Send + Sync + 'static> FromDisconnectParts<A> for HttpExtension<T>

Source

impl<A: Adapter, T: Clone + Send + Sync + 'static> FromDisconnectParts<A> for MaybeExtension<T>

Available on crate feature extensions only.
Source

impl<A: Adapter, T: Clone + Send + Sync + 'static> FromDisconnectParts<A> for MaybeHttpExtension<T>

Source

impl<A: Adapter, T: Clone + Send + Sync + 'static> FromDisconnectParts<A> for State<T>

Available on crate feature state only.